//
//  SALoginViewController.m
//  SocialAuth
//
//  Created by Dan Fleming on 10/24/12.
//  Copyright (c) 2012 Dan Fleming. All rights reserved.
//

#import "SALoginViewController.h"

@interface SASocialItem : NSObject
@property(nonatomic,strong) NSString *title;
@property(nonatomic,strong) NSString *subtitle;
@property(nonatomic,strong) NSString *imageName;
@property(nonatomic,strong) NSDictionary *config;
@property(nonatomic,strong) NSDictionary *data;
@property(nonatomic,strong) NSError *error;
@end

@implementation SASocialItem
@synthesize title, subtitle, imageName, config, data, error;
@end

@interface SALoginViewController()
@property(nonatomic,strong) NSMutableArray *socialArray;
@end

@implementation SALoginViewController

#pragma mark -
#pragma mark View lifecycle

-(id) initWithStyle:(UITableViewStyle)style {
    if(self = [super initWithStyle:style]){
        self.title = @"SocialAuth";
        self.socialArray = [NSMutableArray arrayWithCapacity:6];
        
        // FACEBOOK
        SASocialItem *item = [SASocialItem new];
        item.title = @"Facebook";
        item.subtitle = @"Facebook-iOS-SDK";
        item.imageName = @"facebook.png";
        //[self.socialArray addObject:item];
        
        
        // FOURSQUARE
        item = [SASocialItem new];
        item.title = @"Foursquare";
        item.subtitle = @"OAuth 2.0";
        item.imageName = @"foursquare.png";
        item.config = [SocialAuth extendBaseConfiguration:kSAServiceFoursquare withDictionary: @{
            kSAConfigOAuth2ClientID: @"WBOXT4BNTZKDEKMLJY1YFHKW3W5UYOAWVC34F3MR2BPVW3KH",
            kSAConfigOAuth2RedirectURL: @"http://dfle.ms/projects/socialauth/foursquare"
        }];
        [self.socialArray addObject:item];
        
        
        // INSTAGRAM
        item = [SASocialItem new];
        item.title = @"Instagram";
        item.subtitle = @"OAuth 2.0";
        item.imageName = @"instagram.png";
        item.config = [SocialAuth extendBaseConfiguration:kSAServiceInstagram withDictionary: @{
            kSAConfigOAuth2ClientID: @"a09bdb8b50db4491970a693cb7cfa605",
            kSAConfigOAuth2RedirectURL: @"http://dfle.ms/projects/socialauth/instagram",
            kSAConfigOAuth2Scope: @"comments"
        }];
        [self.socialArray addObject:item];
        
        
        // GITHUB
        item = [SASocialItem new];
        item.title = @"Github";
        item.subtitle = @"OAuth 2.0";
        item.imageName = @"github.png";
        item.config = [SocialAuth extendBaseConfiguration:kSAServiceGithub withDictionary: @{
            kSAConfigOAuth2ClientID: @"14770b30fcee3cd7790d",
            kSAConfigOAuth2RedirectURL: @"http://dfle.ms/projects/socialauth/github"
        }];
        [self.socialArray addObject:item];
        
        
        // TWITTER
        item = [SASocialItem new];
        item.title = @"Twitter";
        item.subtitle = @"OAuth 1.0a";
        item.imageName = @"twitter.png";
        item.config = [SocialAuth extendBaseConfiguration:kSAServiceTwitterWebAuth withDictionary: @{
            kSAConfigOAuth1ConsumerKey: @"zFMXeoNwDTVQBHLSSX43yA",
            kSAConfigOAuth1ConsumerSecret: @"qT6MWI43tMdsT6xCGB1fc2pNMJ99MCQOMI66zStF5k",
            kSAConfigOAuth1RedirectURL: @"http://dfle.ms/projects/socialauth/twitter"
        }];
        [self.socialArray addObject:item];
        
        
        // TUMBLR
        item = [SASocialItem new];
        item.title = @"Tumblr";
        item.subtitle = @"OAuth 1.0a";
        item.imageName = @"tumblr";
        item.config = [SocialAuth extendBaseConfiguration:kSAServiceTumblr withDictionary: @{
            kSAConfigOAuth1ConsumerKey: @"lN9xvsvsHQkFdwSi255g8aCKoLYDfVPUV2Qr1ebDLZYFx30mmf",
            kSAConfigOAuth1ConsumerSecret: @"uzxGcGQiYQ6Ed5zZXQUFzQTTshbjRX3WoSbLOlbcZ6iPKvW8Kb",
            kSAConfigOAuth1RedirectURL: @"http://dfle.ms/projects/socialauth/tumblr"
        }];
        [self.socialArray addObject:item];
        
        
        // FLICKR
        item = [SASocialItem new];
        item.title = @"Flickr";
        item.subtitle = @"OAuth 1.0a";
        item.imageName = @"flickr.png";
        //[self.socialArray addObject:item];
    }
    return self;
}

-(void) viewDidLoad {
    [super viewDidLoad];
    
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    
    UIView *bgView = [[UIView alloc] init];
    bgView.backgroundColor = [UIColor scrollViewTexturedBackgroundColor];
    self.tableView.backgroundView = bgView;
}

-(void) viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.tableView deselectRowAtIndexPath:self.tableView.indexPathForSelectedRow animated:animated];
}

#pragma mark -
#pragma mark Logic

-(SASocialItem*) itemAtIndexPath:(NSIndexPath *)indexPath {
    return [self.socialArray objectAtIndex:indexPath.row];
}

-(SASocialItem*) itemWithConfigName:(NSString*)name {
    if(name){
        for(SASocialItem *item in self.socialArray){
            if([name isEqualToString:item.config[kSAConfigInternalName]])
                return item;
        }
    }
    return nil;
}

-(void) authenticate:(SASocialItem*)item {
    SAViewController *vc = [SocialAuth createAuthViewController:item.config delegate:self];
    [self.navigationController pushViewController:vc animated:YES];
}

-(void) showInfo:(SASocialItem*)item {
    if(item.data){
        UIAlertView * alert = [[UIAlertView alloc] initWithTitle:@"Data" message:item.data.description delegate:self cancelButtonTitle:@"Hide" otherButtonTitles:nil];
        [alert show];
    }
}

#pragma mark -
#pragma mark SAAuthenticationDelegate methods

-(void) clientAuthentication:(NSDictionary*)config didSucceed:(NSDictionary*)data {
    NSLog(@"Auth Succeeded: %@", data);
    SASocialItem *item = [self itemWithConfigName:config[kSAConfigInternalName]];
    if(item){
        item.error = nil;
        item.data = data;
        [self.tableView reloadData];
    }
}

-(void) clientAuthentication:(NSDictionary*)config didFail:(NSError*)error {
    NSLog(@"Auth Failed: %@", error);
    SASocialItem *item = [self itemWithConfigName:config[kSAConfigInternalName]];
    if(item){
        item.error = error;
        item.data = nil;
        [self.tableView reloadData];
    }
}

-(void) clientAuthenticationWasCancelled:(NSDictionary*)config {
    NSLog(@"Auth Cancelled");
    SASocialItem *item = [self itemWithConfigName:config[kSAConfigInternalName]];
    if(item){
        item.error = nil;
        item.data = nil;
        [self.tableView reloadData];
    }
}

#pragma mark -
#pragma mark UITableViewDataSource methods

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"derp"];
    if(cell == nil){
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"derp"];
        cell.selectionStyle = UITableViewCellSelectionStyleGray;
        cell.textLabel.textColor = [UIColor colorWithRed:0.243 green:0.255 blue:0.298 alpha:1];
    }
    
    SASocialItem *item = [self itemAtIndexPath:indexPath];
    cell.textLabel.text = item.title;
    cell.detailTextLabel.text = item.subtitle;
    cell.imageView.image = [UIImage imageNamed:item.imageName];
    
    // Show success/error in accessory
    if(item.error || item.data){
        UIButton *btn = [[UIButton alloc] init];
        UIImage *img = [UIImage imageNamed:(item.error ? @"red-x.png" : @"green-check.png")];
        btn.frame = CGRectMake(0,0,img.size.width,img.size.height);
        [btn setBackgroundImage:img forState:UIControlStateNormal];
        [btn addTarget:self action:@selector(accessoryButtonTapped:withEvent:) forControlEvents:UIControlEventTouchUpInside];
        cell.accessoryView = btn;
    }else{
        cell.accessoryView = nil;
    }
    
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.socialArray.count;
}

#pragma mark -
#pragma mark UITableViewDelegate methods

-(CGFloat) tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 54.f;
}

-(void) tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self authenticate:[self itemAtIndexPath:indexPath]];
}

// Workaround for custom accessory buttons as a passthrough to the delegate method
- (void) accessoryButtonTapped:(UIControl*)button withEvent:(UIEvent*)event {
    NSIndexPath * indexPath = [self.tableView indexPathForRowAtPoint:[[[event touchesForView: button] anyObject] locationInView: self.tableView]];
    if (indexPath != nil)
        [self.tableView.delegate tableView: self.tableView accessoryButtonTappedForRowWithIndexPath: indexPath];
}

-(void) tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
    [self showInfo:[self itemAtIndexPath:indexPath]];
}

@end
