//
//  SocialAuth.h
//  SocialAuth
//
//  Created by Dan Fleming on 10/24/12.
//  Copyright (c) 2012 Dan Fleming. All rights reserved.
//

#import "SocialAuth.h"
#import "SAViewController.h"

// SocialAuth

@implementation SocialAuth

// One-time initialization of built-in services that users can extend

+(NSDictionary*) baseConfigurations {
    static dispatch_once_t once;
    static NSDictionary *_sharedBaseConfigurations;
    dispatch_once(&once, ^{
        NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:5];
        
        // Twitter web authentication
        dict[kSAServiceTwitterWebAuth] = @{
            kSAConfigInternalName:          kSAServiceTwitterWebAuth,
            kSAConfigAuthenticatonType:     kSAAuthenticationTypeOAuth1,
            kSAConfigServiceTitle:          @"Twitter",
            kSAConfigWebBaseDomain:         @"twitter.com",
            kSAConfigWebAuthenticationURL:  @"https://api.twitter.com/oauth/authorize",
            kSAConfigOAuth1RequestTokenURL: @"https://api.twitter.com/oauth/request_token",
            kSAConfigOAuth1AccessTokenURL:  @"https://api.twitter.com/oauth/access_token"
        };
        
        // Tumblr
        dict[kSAServiceTumblr] = @{
            kSAConfigInternalName:          kSAServiceTumblr,
            kSAConfigAuthenticatonType:     kSAAuthenticationTypeOAuth1,
            kSAConfigServiceTitle:          @"Tumblr",
            kSAConfigWebBaseDomain:         @"tumblr.com",
            kSAConfigWebAuthenticationURL:  @"http://www.tumblr.com/oauth/authorize",
            kSAConfigOAuth1RequestTokenURL: @"http://www.tumblr.com/oauth/request_token",
            kSAConfigOAuth1AccessTokenURL:  @"http://www.tumblr.com/oauth/access_token"
        };
        
        // Foursquare
        dict[kSAServiceFoursquare] = @{
            kSAConfigInternalName:         kSAServiceFoursquare,
            kSAConfigAuthenticatonType:    kSAAuthenticationTypeOAuth2,
            kSAConfigServiceTitle:         @"Foursquare",
            kSAConfigWebBaseDomain:        @"foursquare.com",
            kSAConfigWebAuthenticationURL: @"https://foursquare.com/oauth2/authenticate",
        };
        
        // Instagram
        dict[kSAServiceInstagram] = @{
            kSAConfigInternalName:         kSAServiceInstagram,
            kSAConfigAuthenticatonType:    kSAAuthenticationTypeOAuth2,
            kSAConfigServiceTitle:         @"Instagram",
            kSAConfigWebBaseDomain:        @"instagram.com",
            kSAConfigWebAuthenticationURL: @"https://api.instagram.com/oauth/authorize",
        };
        
        // Github
        dict[kSAServiceGithub] = @{
            kSAConfigInternalName:         kSAServiceGithub,
            kSAConfigAuthenticatonType:    kSAAuthenticationTypeOAuth2,
            kSAConfigServiceTitle:         @"Github",
            kSAConfigWebBaseDomain:        @"github.com",
            kSAConfigWebAuthenticationURL: @"https://github.com/login/oauth/authorize",
        };
        
        _sharedBaseConfigurations = dict;
    });
    return _sharedBaseConfigurations;
}

+(NSDictionary*) extendBaseConfiguration:(NSString*)configurationName withDictionary:(NSDictionary*)dict {
    NSDictionary *base = [[self baseConfigurations] objectForKey:configurationName];
    if(base == nil)
        [NSException raise:@"Invalid base configuration" format:@"%@ is not a supported configuration name.", configurationName];
    // TODO: do a "true" dictionary merge - add to dictionary category
    NSMutableDictionary *newDict = [base mutableCopy];
    [newDict addEntriesFromDictionary:dict];
    return newDict;
}

+(SAViewController*) createAuthViewController:(NSDictionary*)config delegate:(id<SAAuthenticationDelegate>)delegate {
    NSString *type = config[kSAConfigAuthenticatonType];
    if(type){
        if([kSAAuthenticationTypeOAuth1 isEqualToString:type]){
            return [[SAOAuth1WebViewController alloc] initWithConfig:config delegate:delegate];
        }else if([kSAAuthenticationTypeOAuth2 isEqualToString:type]){
            return [[SAOAuth2WebViewController alloc] initWithConfig:config delegate:delegate];
        }
    }
    return nil;
}

@end
